@@ -7,17 +7,17 @@ build, and optionally (but by-default) mounted at boot in the final system. The
 mountpoints are sorted on their position in the filesystem hierarchy so the
 order in the recipe does not matter.
 
- # Yaml syntax:
- - action: image-partition
-   imagename: image_name
-   imagesize: size
-   partitiontype: gpt
-   diskid: string
-   gpt_gap: offset
-   partitions:
-     <list of partitions>
-   mountpoints:
-     <list of mount points>
+	# Yaml syntax:
+	- action: image-partition
+	  imagename: image_name
+	  imagesize: size
+	  partitiontype: gpt
+	  diskid: string
+	  gpt_gap: offset
+	  partitions:
+	    <list of partitions>
+	  mountpoints:
+	    <list of mount points>
 
 Mandatory properties:
 
@@ -45,20 +45,21 @@ should be in GUID format (e.g.: '00002222-4444-6666-AAAA-BBBBCCCCFFFF' where eac
 character is an hexadecimal digit). For 'msdos' partition table, 'diskid' should be
 a 32 bits hexadecimal number (e.g. '1234ABCD' without any dash separator).
 
-   # Yaml syntax for partitions:
-   partitions:
-     - name: partition name
-	   partlabel: partition label
-	   fs: filesystem
-	   fslabel: filesystem label
-	   start: offset
-	   end: offset
-	   features: list of filesystem features
-	   extendedoptions: list of filesystem extended options
-	   flags: list of flags
-	   fsck: bool
-	   fsuuid: string
-	   partuuid: string
+	   # Yaml syntax for partitions:
+	   partitions:
+	     - name: partition name
+		   partlabel: partition label
+		   fs: filesystem
+		   fslabel: filesystem label
+		   start: offset
+		   end: offset
+		   features: list of filesystem features
+		   extendedoptions: list of filesystem extended options
+		   flags: list of flags
+		   fsck: bool
+		   fsuuid: string
+		   partuuid: string
+		   partattrs: list of partition attribute bits to set
 
 Mandatory properties:
 
@@ -102,6 +103,16 @@ for partition.
 - flags -- list of additional flags for partition compatible with parted(8)
 'set' command.
 
+- partattrs -- list of GPT partition attribute bits to set, as defined in
+https://uefi.org/specs/UEFI/2.10/05_GUID_Partition_Table_Format.html#defined-gpt-partition-entry-attributes.
+Bit 0: "Required Partition", bit 1: "No Block IO Protocol", bit 2: "Legacy BIOS
+Bootable". Bits 3-47 are reserved. Bits 48-63 are GUID specific. For example,
+ChromeOS Kernel partitions (GUID=fe3a2a5d-4f32-41a7-b725-accc3285a309) use bit
+56 for "successful boot" and bits 48-51 for "priority", where 0 means not
+bootable, thus bits 56 and 48 need to be set through this property in order to
+be able to boot a ChromeOS Kernel partition on a Chromebook, like so:
+'partattrs: [56, 48]'.
+
 - fsck -- if set to `false` -- then set fs_passno (man fstab) to 0 meaning no filesystem
 checks in boot time. By default is set to `true` allowing checks on boot.
 
@@ -117,12 +128,12 @@ and data.
 - extendedoptions -- list of additional filesystem extended options which need
 to be enabled for the partition.
 
-   # Yaml syntax for mount points:
-   mountpoints:
-     - mountpoint: path
-	   partition: partition label
-	   options: list of options
-	   buildtime: bool
+	   # Yaml syntax for mount points:
+	   mountpoints:
+	     - mountpoint: path
+		   partition: partition label
+		   options: list of options
+		   buildtime: bool
 
 Mandatory properties:
 
@@ -143,27 +154,27 @@ to define a `mountpoint` path which is temporary and unique for the image,
 for example: `/mnt/temporary_mount`.
 Defaults to false.
 
- # Layout example for Raspberry PI 3:
- - action: image-partition
-   imagename: "debian-rpi3.img"
-   imagesize: 1GB
-   partitiontype: msdos
-   mountpoints:
-     - mountpoint: /
-       partition: root
-     - mountpoint: /boot/firmware
-       partition: firmware
-       options: [ x-systemd.automount ]
-   partitions:
-     - name: firmware
-       fs: vfat
-       start: 0%
-       end: 64MB
-     - name: root
-       fs: ext4
-       start: 64MB
-       end: 100%
-       flags: [ boot ]
+	# Layout example for Raspberry PI 3:
+	- action: image-partition
+	  imagename: "debian-rpi3.img"
+	  imagesize: 1GB
+	  partitiontype: msdos
+	  mountpoints:
+	    - mountpoint: /
+	      partition: root
+	    - mountpoint: /boot/firmware
+	      partition: firmware
+	      options: [ x-systemd.automount ]
+	  partitions:
+	    - name: firmware
+	      fs: vfat
+	      start: 0%
+	      end: 64MB
+	    - name: root
+	      fs: ext4
+	      start: 64MB
+	      end: 100%
+	      flags: [ boot ]
 */
 package actions
 
@@ -172,20 +183,20 @@ import (
 	"errors"
 	"fmt"
 	"github.com/docker/go-units"
+	"github.com/freddierice/go-losetup/v2"
 	"github.com/go-debos/fakemachine"
 	"github.com/google/uuid"
-	"github.com/freddierice/go-losetup/v2"
 	"log"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
+	"regexp"
 	"sort"
 	"strconv"
 	"strings"
 	"syscall"
 	"time"
-	"regexp"
 
 	"github.com/go-debos/debos"
 )
@@ -196,6 +207,7 @@ type Partition struct {
 	PartLabel       string
 	FSLabel         string
 	PartType        string
+	PartAttrs       []string
 	PartUUID        string
 	Start           string
 	End             string
@@ -203,7 +215,7 @@ type Partition struct {
 	Flags           []string
 	Features        []string
 	ExtendedOptions []string
-	Fsck            bool "fsck"
+	Fsck            bool `yaml:"fsck"`
 	FSUUID          string
 }
 
@@ -219,12 +231,15 @@ type imageLocker struct {
 	fd *os.File
 }
 
-func lockImage(context *debos.DebosContext) (*imageLocker, error) {
+func lockImage(context *debos.Context) (*imageLocker, error) {
 	fd, err := os.Open(context.Image)
 	if err != nil {
 		return nil, err
 	}
-	syscall.Flock(int(fd.Fd()), syscall.LOCK_EX)
+	if err := syscall.Flock(int(fd.Fd()), syscall.LOCK_EX); err != nil {
+		fd.Close()
+		return nil, fmt.Errorf("failed to lock image: %w", err)
+	}
 	return &imageLocker{fd: fd}, nil
 }
 
@@ -238,7 +253,7 @@ type ImagePartitionAction struct {
 	ImageSize        string
 	PartitionType    string
 	DiskID           string
-	GptGap           string "gpt_gap"
+	GptGap           string `yaml:"gpt_gap"`
 	Partitions       []Partition
 	Mountpoints      []Mountpoint
 	size             int64
@@ -256,51 +271,49 @@ func (p *Partition) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	return nil
 }
 
-func (i *ImagePartitionAction) generateFSTab(context *debos.DebosContext) error {
+func (i *ImagePartitionAction) generateFSTab(context *debos.Context) error {
 	context.ImageFSTab.Reset()
 
 	for _, m := range i.Mountpoints {
 		options := []string{"defaults"}
 		options = append(options, m.Options...)
-		if m.Buildtime == true {
+		if m.Buildtime {
 			/* Do not need to add mount point into fstab */
 			continue
 		}
 		if m.part.FSUUID == "" {
-			return fmt.Errorf("Missing fs UUID for partition %s!?!", m.part.Name)
+			return fmt.Errorf("missing fs UUID for partition %s", m.part.Name)
 		}
 
-		fs_passno := 0
+		fsPassno := 0
 
 		if m.part.Fsck {
 			if m.Mountpoint == "/" {
-				fs_passno = 1
+				fsPassno = 1
 			} else {
-				fs_passno = 2
+				fsPassno = 2
 			}
 		}
 
 		fsType := m.part.FS
 		switch m.part.FS {
-			case "fat", "fat12", "fat16", "fat32", "msdos":
-				fsType = "vfat"
-			default:
-				break
+		case "fat", "fat12", "fat16", "fat32", "msdos":
+			fsType = "vfat"
 		}
 
 		context.ImageFSTab.WriteString(fmt.Sprintf("UUID=%s\t%s\t%s\t%s\t0\t%d\n",
 			m.part.FSUUID, m.Mountpoint, fsType,
-			strings.Join(options, ","), fs_passno))
+			strings.Join(options, ","), fsPassno))
 	}
 
 	return nil
 }
 
-func (i *ImagePartitionAction) generateKernelRoot(context *debos.DebosContext) error {
+func (i *ImagePartitionAction) generateKernelRoot(context *debos.Context) error {
 	for _, m := range i.Mountpoints {
 		if m.Mountpoint == "/" {
 			if m.part.FSUUID == "" {
-				return errors.New("No fs UUID for root partition !?!")
+				return errors.New("no fs UUID for root partition")
 			}
 			context.ImageKernelRoot = fmt.Sprintf("root=UUID=%s", m.part.FSUUID)
 			break
@@ -310,7 +323,7 @@ func (i *ImagePartitionAction) generateKernelRoot(context *debos.DebosContext) e
 	return nil
 }
 
-func (i ImagePartitionAction) getPartitionDevice(number int, context debos.DebosContext) string {
+func (i ImagePartitionAction) getPartitionDevice(number int, context debos.Context) string {
 	/* Always look up canonical device as udev might not generate the by-id
 	 * symlinks while there is an flock on /dev/vda */
 	device, _ := filepath.EvalSymlinks(context.Image)
@@ -326,12 +339,11 @@ func (i ImagePartitionAction) getPartitionDevice(number int, context debos.Debos
 	last := device[len(device)-1]
 	if last >= '0' && last <= '9' {
 		return fmt.Sprintf("%s%s%d", device, suffix, number)
-	} else {
-		return fmt.Sprintf("%s%d", device, number)
 	}
+	return fmt.Sprintf("%s%d", device, number)
 }
 
-func (i *ImagePartitionAction) triggerDeviceNodes(context *debos.DebosContext) error {
+func (i *ImagePartitionAction) triggerDeviceNodes(context *debos.Context) error {
 	err := debos.Command{}.Run("udevadm", "udevadm", "trigger", "--settle", context.Image)
 	if err != nil {
 		log.Printf("Failed to trigger device nodes")
@@ -341,7 +353,7 @@ func (i *ImagePartitionAction) triggerDeviceNodes(context *debos.DebosContext) e
 	return nil
 }
 
-func (i ImagePartitionAction) PreMachine(context *debos.DebosContext, m *fakemachine.Machine,
+func (i ImagePartitionAction) PreMachine(context *debos.Context, m *fakemachine.Machine,
 	args *[]string) error {
 	imagePath := path.Join(context.Artifactdir, i.ImageName)
 	image, err := m.CreateImage(imagePath, i.size)
@@ -354,7 +366,7 @@ func (i ImagePartitionAction) PreMachine(context *debos.DebosContext, m *fakemac
 	return nil
 }
 
-func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosContext) error {
+func (i ImagePartitionAction) formatPartition(p *Partition, context debos.Context) error {
 	label := fmt.Sprintf("Formatting partition %d", p.number)
 	path := i.getPartitionDevice(p.number, context)
 
@@ -446,7 +458,7 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 	if p.FS != "none" && p.FSUUID == "" {
 		uuid, err := exec.Command("blkid", "-o", "value", "-s", "UUID", "-p", "-c", "none", path).Output()
 		if err != nil {
-			return fmt.Errorf("Failed to get uuid: %s", err)
+			return fmt.Errorf("failed to get uuid: %w", err)
 		}
 		p.FSUUID = strings.TrimSpace(string(uuid[:]))
 	}
@@ -454,16 +466,16 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 	return nil
 }
 
-func (i *ImagePartitionAction) PreNoMachine(context *debos.DebosContext) error {
+func (i *ImagePartitionAction) PreNoMachine(context *debos.Context) error {
 	imagePath := path.Join(context.Artifactdir, i.ImageName)
 	img, err := os.OpenFile(imagePath, os.O_WRONLY|os.O_CREATE, 0666)
 	if err != nil {
-		return fmt.Errorf("Couldn't open image file: %v", err)
+		return fmt.Errorf("couldn't open image file: %w", err)
 	}
 
 	err = img.Truncate(i.size)
 	if err != nil {
-		return fmt.Errorf("Couldn't resize image file: %v", err)
+		return fmt.Errorf("couldn't resize image file: %w", err)
 	}
 
 	img.Close()
@@ -480,7 +492,7 @@ func (i *ImagePartitionAction) PreNoMachine(context *debos.DebosContext) error {
 	}
 
 	if err != nil {
-		return fmt.Errorf("Failed to setup loop device: %v", err)
+		return fmt.Errorf("failed to setup loop device: %w", err)
 	}
 
 	// go-losetup doesn't provide a way to change the loop device sector size
@@ -499,7 +511,7 @@ func (i *ImagePartitionAction) PreNoMachine(context *debos.DebosContext) error {
 	return nil
 }
 
-func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
+func (i ImagePartitionAction) Run(context *debos.Context) error {
 	/* On certain disk device events udev will call the BLKRRPART ioctl to
 	 * re-read the partition table. This will cause the partition devices
 	 * (e.g. vda3) to temporarily disappear while the rescanning happens.
@@ -524,7 +536,7 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 		}
 	}
 
-	for idx, _ := range i.Partitions {
+	for idx := range i.Partitions {
 		p := &i.Partitions[idx]
 
 		if p.PartLabel == "" {
@@ -587,6 +599,26 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 			}
 		}
 
+		if len(p.PartAttrs) > 0 {
+			/* Convert bits numbers to bits names due to a libfdisk's limitation
+			 * https://github.com/util-linux/util-linux/issues/3353
+			 */
+			for idx, attr := range p.PartAttrs {
+				switch attr {
+				case "0":
+					p.PartAttrs[idx] = "RequiredPartition"
+				case "1":
+					p.PartAttrs[idx] = "NoBlockIOProtocol"
+				case "2":
+					p.PartAttrs[idx] = "LegacyBIOSBootable"
+				}
+			}
+			err = debos.Command{}.Run("sfdisk", "sfdisk", "--part-attrs", context.Image, fmt.Sprintf("%d", p.number), strings.Join(p.PartAttrs, ","))
+			if err != nil {
+				return err
+			}
+		}
+
 		/* PartUUID will only be set for gpt partitions */
 		if len(p.PartUUID) > 0 {
 			err = debos.Command{}.Run("sfdisk", "sfdisk", "--part-uuid", context.Image, fmt.Sprintf("%d", p.number), p.PartUUID)
@@ -609,11 +641,13 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 
 		devicePath := i.getPartitionDevice(p.number, *context)
 		context.ImagePartitions = append(context.ImagePartitions,
-			debos.Partition{p.Name, devicePath})
+			debos.Partition{Name: p.Name, DevicePath: devicePath})
 	}
 
 	context.ImageMntDir = path.Join(context.Scratchdir, "mnt")
-	os.MkdirAll(context.ImageMntDir, 0755)
+	if err := os.MkdirAll(context.ImageMntDir, 0755); err != nil {
+		return fmt.Errorf("failed to create mount directory: %w", err)
+	}
 
 	// sort mountpoints based on position in filesystem hierarchy
 	sort.SliceStable(i.Mountpoints, func(a, b int) bool {
@@ -621,10 +655,10 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 		mntB := i.Mountpoints[b].Mountpoint
 
 		// root should always be mounted first
-		if (mntA == "/") {
+		if mntA == "/" {
 			return true
 		}
-		if (mntB == "/") {
+		if mntB == "/" {
 			return false
 		}
 
@@ -640,17 +674,17 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 	for _, m := range i.Mountpoints {
 		dev := i.getPartitionDevice(m.part.number, *context)
 		mntpath := path.Join(context.ImageMntDir, m.Mountpoint)
-		os.MkdirAll(mntpath, 0755)
+		if err := os.MkdirAll(mntpath, 0755); err != nil {
+			return fmt.Errorf("failed to create mountpoint %s: %w", mntpath, err)
+		}
 		fsType := m.part.FS
 		switch m.part.FS {
-			case "fat", "fat12", "fat16", "fat32", "msdos":
-				fsType = "vfat"
-			default:
-				break
+		case "fat", "fat12", "fat16", "fat32", "msdos":
+			fsType = "vfat"
 		}
 		err = syscall.Mount(dev, mntpath, fsType, 0, "")
 		if err != nil {
-			return fmt.Errorf("%s mount failed: %v", m.part.Name, err)
+			return fmt.Errorf("%s mount failed: %w", m.part.Name, err)
 		}
 	}
 	lock.unlock()
@@ -668,11 +702,13 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 	/* Now that all partitions are created (re)trigger all udev events for
 	 * the image file to make sure everything is in a reasonable state
 	 */
-	i.triggerDeviceNodes(context)
+	if err := i.triggerDeviceNodes(context); err != nil {
+		return fmt.Errorf("failed to trigger device nodes: %w", err)
+	}
 	return nil
 }
 
-func (i ImagePartitionAction) Cleanup(context *debos.DebosContext) error {
+func (i ImagePartitionAction) Cleanup(context *debos.Context) error {
 	for idx := len(i.Mountpoints) - 1; idx >= 0; idx-- {
 		m := i.Mountpoints[idx]
 		mntpath := path.Join(context.ImageMntDir, m.Mountpoint)
@@ -682,11 +718,12 @@ func (i ImagePartitionAction) Cleanup(context *debos.DebosContext) error {
 			log.Printf("Unmount failure can cause images being incomplete!")
 			return err
 		}
-		if m.Buildtime == true {
+		if m.Buildtime {
 			if err = os.Remove(mntpath); err != nil {
 				log.Printf("Failed to remove temporary mount point %s: %s", m.Mountpoint, err)
 
-				if err.(*os.PathError).Err.Error() == "read-only file system" {
+				var pathErr *os.PathError
+				if errors.As(err, &pathErr) && pathErr.Err.Error() == "read-only file system" {
 					continue
 				}
 
@@ -719,7 +756,7 @@ func (i ImagePartitionAction) Cleanup(context *debos.DebosContext) error {
 	return nil
 }
 
-func (i ImagePartitionAction) PostMachineCleanup(context *debos.DebosContext) error {
+func (i ImagePartitionAction) PostMachineCleanup(context *debos.Context) error {
 	image := path.Join(context.Artifactdir, i.ImageName)
 	/* Remove the image in case of any action failure */
 	if context.State != debos.Success {
@@ -732,10 +769,9 @@ func (i ImagePartitionAction) PostMachineCleanup(context *debos.DebosContext) er
 	return nil
 }
 
-func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
-
+func (i *ImagePartitionAction) Verify(_ *debos.Context) error {
 	if i.PartitionType == "msdos" {
-		for idx, _ := range i.Partitions {
+		for idx := range i.Partitions {
 			p := &i.Partitions[idx]
 
 			if idx == 3 && len(i.Partitions) > 4 {
@@ -746,8 +782,8 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 				part.number = idx + 1
 				part.Name = name
 				part.Start = p.Start
-				tmp_n := len(i.Partitions) - 1
-				tmp := &i.Partitions[tmp_n]
+				tmpN := len(i.Partitions) - 1
+				tmp := &i.Partitions[tmpN]
 				part.End = tmp.End
 				part.FS = "none"
 
@@ -755,7 +791,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 				i.Partitions[idx] = part
 
 				num := 1
-				for idx, _ := range i.Partitions {
+				for idx := range i.Partitions {
 					p := &i.Partitions[idx]
 					p.number = num
 					num++
@@ -772,7 +808,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 		// Just check if it contains correct value
 		_, err := units.FromHumanSize(i.GptGap)
 		if err != nil {
-			return fmt.Errorf("Failed to parse GPT offset: %s", i.GptGap)
+			return fmt.Errorf("failed to parse image size: %s", i.GptGap)
 		}
 	}
 
@@ -781,12 +817,12 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 		case "gpt":
 			_, err := uuid.Parse(i.DiskID)
 			if err != nil {
-				return fmt.Errorf("Incorrect disk GUID %s", i.DiskID)
+				return fmt.Errorf("incorrect disk GUID %s", i.DiskID)
 			}
 		case "msdos":
 			_, err := hex.DecodeString(i.DiskID)
 			if err != nil || len(i.DiskID) != 8 {
-				return fmt.Errorf("Incorrect disk ID %s, should be 32-bit hexadecimal number", i.DiskID)
+				return fmt.Errorf("incorrect disk ID %s, should be 32-bit hexadecimal number", i.DiskID)
 			}
 			// Add 0x prefix
 			i.DiskID = "0x" + i.DiskID
@@ -794,19 +830,19 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 	}
 
 	num := 1
-	for idx, _ := range i.Partitions {
-		var maxLength int = 0
+	for idx := range i.Partitions {
+		var maxLength = 0
 		p := &i.Partitions[idx]
 		p.number = num
 		num++
 		if p.Name == "" {
-			return fmt.Errorf("Partition without a name")
+			return fmt.Errorf("partition without a name")
 		}
 
 		// check for duplicate partition names
 		for j := idx + 1; j < len(i.Partitions); j++ {
 			if i.Partitions[j].Name == p.Name {
-				return fmt.Errorf("Partition %s already exists", p.Name)
+				return fmt.Errorf("partition %s already exists", p.Name)
 			}
 		}
 
@@ -815,20 +851,20 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 			case "btrfs", "ext2", "ext3", "ext4", "xfs":
 				_, err := uuid.Parse(p.FSUUID)
 				if err != nil {
-					return fmt.Errorf("Incorrect UUID %s", p.FSUUID)
+					return fmt.Errorf("incorrect UUID %s", p.FSUUID)
 				}
 			case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
 				_, err := hex.DecodeString(p.FSUUID)
 				if err != nil || len(p.FSUUID) != 8 {
-					return fmt.Errorf("Incorrect UUID %s, should be 32-bit hexadecimal number", p.FSUUID)
+					return fmt.Errorf("incorrect UUID %s, should be 32-bit hexadecimal number", p.FSUUID)
 				}
 			default:
-				return fmt.Errorf("Setting the UUID is not supported for filesystem %s", p.FS)
+				return fmt.Errorf("setting the UUID is not supported for filesystem %s", p.FS)
 			}
 		}
 
 		if i.PartitionType != "gpt" && p.PartLabel != "" {
-			return fmt.Errorf("Can only set partition partlabel on GPT filesystem")
+			return fmt.Errorf("can only set partition partlabel on GPT filesystem")
 		}
 
 		if len(p.PartUUID) > 0 {
@@ -836,10 +872,10 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 			case "gpt":
 				_, err := uuid.Parse(p.PartUUID)
 				if err != nil {
-					return fmt.Errorf("Incorrect partition UUID %s", p.PartUUID)
+					return fmt.Errorf("incorrect partition UUID %s", p.PartUUID)
 				}
 			default:
-				return fmt.Errorf("Setting the partition UUID is not supported for %s", i.PartitionType)
+				return fmt.Errorf("setting the partition UUID is not supported for %s", i.PartitionType)
 			}
 		}
 
@@ -856,15 +892,22 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 			}
 		}
 
+		for _, bitStr := range p.PartAttrs {
+			bit, err := strconv.ParseInt(bitStr, 0, 0)
+			if err != nil || bit < 0 || bit > 2 && bit < 48 || bit > 63 {
+				return fmt.Errorf("partition attribute bit '%s' outside of valid range (0-2, 48-63)", bitStr)
+			}
+		}
+
 		if p.Start == "" {
-			return fmt.Errorf("Partition %s missing start", p.Name)
+			return fmt.Errorf("partition %s missing start", p.Name)
 		}
 		if p.End == "" {
-			return fmt.Errorf("Partition %s missing end", p.Name)
+			return fmt.Errorf("partition %s missing end", p.Name)
 		}
 
 		if p.FS == "" {
-			return fmt.Errorf("Partition %s missing fs type", p.Name)
+			return fmt.Errorf("partition %s missing fs type", p.Name)
 		}
 
 		if p.FSLabel == "" {
@@ -872,39 +915,39 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 		}
 
 		switch p.FS {
-			case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
-				maxLength = 11
-			case "ext2", "ext3", "ext4":
-				maxLength = 16
-			case "btrfs":
-				maxLength = 255
-			case "f2fs":
-				maxLength = 512
-			case "hfs", "hfsplus":
-				maxLength = 255
-			case "xfs":
-				maxLength = 12
-			case "none":
-			default:
-				log.Printf("Warning: setting a fs label for %s is unsupported", p.FS)
+		case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
+			maxLength = 11
+		case "ext2", "ext3", "ext4":
+			maxLength = 16
+		case "btrfs":
+			maxLength = 255
+		case "f2fs":
+			maxLength = 512
+		case "hfs", "hfsplus":
+			maxLength = 255
+		case "xfs":
+			maxLength = 12
+		case "none":
+		default:
+			log.Printf("Warning: setting a fs label for %s is unsupported", p.FS)
 		}
 
 		if maxLength > 0 && len(p.FSLabel) > maxLength {
-		        return fmt.Errorf("fs label for %s '%s' is too long", p.Name, p.FSLabel)
+			return fmt.Errorf("fs label for %s '%s' is too long", p.Name, p.FSLabel)
 		}
 	}
 
-	for idx, _ := range i.Mountpoints {
+	for idx := range i.Mountpoints {
 		m := &i.Mountpoints[idx]
 
 		// check for duplicate mountpoints
 		for j := idx + 1; j < len(i.Mountpoints); j++ {
 			if i.Mountpoints[j].Mountpoint == m.Mountpoint {
-				return fmt.Errorf("Mountpoint %s already exists", m.Mountpoint)
+				return fmt.Errorf("mountpoint %s already exists", m.Mountpoint)
 			}
 		}
 
-		for pidx, _ := range i.Partitions {
+		for pidx := range i.Partitions {
 			p := &i.Partitions[pidx]
 			if m.Partition == p.Name {
 				m.part = p
@@ -912,11 +955,11 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 			}
 		}
 		if m.part == nil {
-			return fmt.Errorf("Couldn't find partition for %s", m.Mountpoint)
+			return fmt.Errorf("couldn't find partition for %s", m.Mountpoint)
 		}
 
 		if strings.ToLower(m.part.FS) == "none" {
-			return fmt.Errorf("Cannot mount %s: filesystem not present", m.Mountpoint)
+			return fmt.Errorf("cannot mount %s: filesystem not present", m.Mountpoint)
 		}
 	}
 
@@ -932,7 +975,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 
 	size, err := getSizeValueFunc(i.ImageSize)
 	if err != nil {
-		return fmt.Errorf("Failed to parse image size: %s", i.ImageSize)
+		return fmt.Errorf("failed to parse image size: %s", i.ImageSize)
 	}
 
 	i.size = size
