@@ -4,20 +4,21 @@ Raw Action
 Directly write a file to the output image at a given offset.
 This is typically useful for bootloaders.
 
- # Yaml syntax:
- - action: raw
-   origin: name
-   source: filename
-   offset: bytes
+	# Yaml syntax:
+	- action: raw
+	  origin: name
+	  source: filename
+	  offset: bytes
 
 Mandatory properties:
 
-- origin -- reference to named file or directory.
-
-- source -- the name of file located in 'origin' to be written into the output image.
+- source -- the name of file to be written into the output image.
 
 Optional properties:
 
+- origin -- reference to named file or directory.
+If not provided, defaults to the recipe directory.
+
 - offset -- offset in bytes or in sector number e.g 256s.
 The sector size is either the recipe header 'sectorsize' or the default 512 sector
 size.
@@ -32,7 +33,6 @@ package actions
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
 	"path"
@@ -52,7 +52,6 @@ type RawAction struct {
 }
 
 func (raw *RawAction) checkDeprecatedSyntax() error {
-
 	// New syntax is based on 'origin' and 'source'
 	// Check if we do not mix new and old syntax
 	// TODO: remove deprecated syntax verification
@@ -61,7 +60,7 @@ func (raw *RawAction) checkDeprecatedSyntax() error {
 		log.Printf("Usage of 'source' and 'path' properties is deprecated.")
 		log.Printf("Please use 'origin' and 'source' properties.")
 		if len(raw.Origin) > 0 {
-			return errors.New("Can't mix 'origin' and 'path'(deprecated option) properties")
+			return errors.New("can't mix 'origin' and 'path'(deprecated option) properties")
 		}
 		if len(raw.Source) == 0 {
 			return errors.New("'source' and 'path' properties can't be empty")
@@ -74,28 +73,33 @@ func (raw *RawAction) checkDeprecatedSyntax() error {
 	return nil
 }
 
-func (raw *RawAction) Verify(context *debos.DebosContext) error {
+func (raw *RawAction) Verify(_ *debos.Context) error {
 	if err := raw.checkDeprecatedSyntax(); err != nil {
 		return err
 	}
 
-	if len(raw.Origin) == 0 || len(raw.Source) == 0 {
-		return errors.New("'origin' and 'source' properties can't be empty")
+	if len(raw.Source) == 0 {
+		return errors.New("'source' property can't be empty")
 	}
 
 	return nil
 }
 
-func (raw *RawAction) Run(context *debos.DebosContext) error {
-	origin, found := context.Origin(raw.Origin)
-	if !found {
-		return fmt.Errorf("Origin `%s` doesn't exist\n", raw.Origin)
+func (raw *RawAction) Run(context *debos.Context) error {
+	origin := context.RecipeDir
+
+	if len(raw.Origin) > 0 {
+		var found bool
+		if origin, found = context.Origin(raw.Origin); !found {
+			return fmt.Errorf("origin `%s` doesn't exist", raw.Origin)
+		}
 	}
+
 	s := path.Join(origin, raw.Source)
-	content, err := ioutil.ReadFile(s)
+	content, err := os.ReadFile(s)
 
 	if err != nil {
-		return fmt.Errorf("Failed to read %s", s)
+		return fmt.Errorf("failed to read %s", s)
 	}
 
 	var devicePath string
@@ -108,7 +112,7 @@ func (raw *RawAction) Run(context *debos.DebosContext) error {
 		}
 
 		if devicePath == "" {
-			return fmt.Errorf("Failed to find partition named %s", raw.Partition)
+			return fmt.Errorf("failed to find partition named %s", raw.Partition)
 		}
 	} else {
 		devicePath = context.Image
@@ -116,11 +120,11 @@ func (raw *RawAction) Run(context *debos.DebosContext) error {
 
 	target, err := os.OpenFile(devicePath, os.O_WRONLY, 0)
 	if err != nil {
-		return fmt.Errorf("Failed to open %s: %v", devicePath, err)
+		return fmt.Errorf("failed to open %s: %w", devicePath, err)
 	}
 	defer target.Close()
 
-	var offset int64 = 0
+	var offset int64
 	if len(raw.Offset) > 0 {
 		sector := false
 		offs := raw.Offset
@@ -130,7 +134,7 @@ func (raw *RawAction) Run(context *debos.DebosContext) error {
 		}
 		offset, err = strconv.ParseInt(offs, 0, 64)
 		if err != nil {
-			return fmt.Errorf("Couldn't parse offset %v", err)
+			return fmt.Errorf("couldn't parse offset %w", err)
 		}
 
 		if sector {
@@ -140,12 +144,12 @@ func (raw *RawAction) Run(context *debos.DebosContext) error {
 
 	bytes, err := target.WriteAt(content, offset)
 	if bytes != len(content) {
-		return fmt.Errorf("Couldn't write complete data %v", err)
+		return fmt.Errorf("couldn't write complete data %w", err)
 	}
 
 	err = target.Sync()
 	if err != nil {
-		return fmt.Errorf("Couldn't sync content %v", err)
+		return fmt.Errorf("couldn't sync content %w", err)
 	}
 
 	return nil
