@@ -5,8 +5,8 @@ import (
 	"crypto/sha256"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"log"
+	"math/rand"
 	"os"
 	"os/exec"
 	"path"
@@ -16,9 +16,9 @@ import (
 type ChrootEnterMethod int
 
 const (
-	CHROOT_METHOD_NONE   = iota // No chroot in use
-	CHROOT_METHOD_NSPAWN        // use nspawn to create the chroot environment
-	CHROOT_METHOD_CHROOT        // use chroot to create the chroot environment
+	ChrootMethodNone   ChrootEnterMethod = iota // No chroot in use
+	ChrootMethodNspawn                          // use nspawn to create the chroot environment
+	ChrootMethodChroot                          // use chroot to create the chroot environment
 )
 
 type Command struct {
@@ -69,8 +69,8 @@ func (w *commandWrapper) flush() {
 	w.out(true)
 }
 
-func NewChrootCommandForContext(context DebosContext) Command {
-	c := Command{Architecture: context.Architecture, Chroot: context.Rootdir, ChrootMethod: CHROOT_METHOD_NSPAWN}
+func NewChrootCommandForContext(context Context) Command {
+	c := Command{Architecture: context.Architecture, Chroot: context.Rootdir, ChrootMethod: ChrootMethodNspawn}
 
 	if context.EnvironVars != nil {
 		for k, v := range context.EnvironVars {
@@ -124,7 +124,7 @@ func (cmd *Command) saveResolvConf() (*[sha256.Size]byte, error) {
 	savedconf := chrootedconf + ".debos"
 	var sum [sha256.Size]byte
 
-	if cmd.ChrootMethod == CHROOT_METHOD_NONE {
+	if cmd.ChrootMethod == ChrootMethodNone {
 		return nil, nil
 	}
 
@@ -136,7 +136,7 @@ func (cmd *Command) saveResolvConf() (*[sha256.Size]byte, error) {
 	}
 
 	/* Expect a relatively small file here */
-	data, err := ioutil.ReadFile(hostconf)
+	data, err := os.ReadFile(hostconf)
 	if err != nil {
 		return nil, err
 	}
@@ -145,7 +145,7 @@ func (cmd *Command) saveResolvConf() (*[sha256.Size]byte, error) {
 
 	sum = sha256.Sum256(out)
 
-	err = ioutil.WriteFile(chrootedconf, out, 0644)
+	err = os.WriteFile(chrootedconf, out, 0644)
 	if err != nil {
 		return nil, err
 	}
@@ -158,7 +158,7 @@ func (cmd *Command) restoreResolvConf(sum *[sha256.Size]byte) error {
 	chrootedconf := path.Join(cmd.Chroot, hostconf)
 	savedconf := chrootedconf + ".debos"
 
-	if cmd.ChrootMethod == CHROOT_METHOD_NONE || sum == nil {
+	if cmd.ChrootMethod == ChrootMethodNone || sum == nil {
 		return nil
 	}
 
@@ -177,14 +177,14 @@ func (cmd *Command) restoreResolvConf(sum *[sha256.Size]byte) error {
 	switch {
 	case mode.IsRegular():
 		// Try to calculate checksum
-		data, err := ioutil.ReadFile(chrootedconf)
+		data, err := os.ReadFile(chrootedconf)
 		if err != nil {
 			return err
 		}
 		currentsum := sha256.Sum256(data)
 
 		// Leave the changed resolv.conf untouched
-		if bytes.Compare(currentsum[:], (*sum)[:]) == 0 {
+		if bytes.Equal(currentsum[:], (*sum)[:]) {
 			// Remove the generated version
 			if err := os.Remove(chrootedconf); err != nil {
 				return err
@@ -210,33 +210,38 @@ func (cmd *Command) restoreResolvConf(sum *[sha256.Size]byte) error {
 }
 
 func (cmd Command) Run(label string, cmdline ...string) error {
-	q := newQemuHelper(cmd)
-	q.Setup()
+	q, err := newQemuHelper(cmd)
+	if err != nil {
+		return err
+	}
+
+	if err := q.Setup(); err != nil {
+		return err
+	}
 	defer q.Cleanup()
 
 	var options []string
 	switch cmd.ChrootMethod {
-	case CHROOT_METHOD_NONE:
+	case ChrootMethodNone:
 		options = cmdline
-	case CHROOT_METHOD_CHROOT:
+	case ChrootMethodChroot:
 		options = append(options, "chroot")
 		options = append(options, cmd.Chroot)
 		options = append(options, cmdline...)
-	case CHROOT_METHOD_NSPAWN:
+	case ChrootMethodNspawn:
 		// We use own resolv.conf handling
 		options = append(options, "systemd-nspawn", "-q")
 		options = append(options, "--resolv-conf=off")
 		options = append(options, "--timezone=off")
 		options = append(options, "--register=no")
+		options = append(options, fmt.Sprintf("--machine=debos-%d", rand.Int63()))
 		options = append(options, "--keep-unit")
 		options = append(options, "--console=pipe")
 		for _, e := range cmd.extraEnv {
 			options = append(options, "--setenv", e)
-
 		}
 		for _, b := range cmd.bindMounts {
 			options = append(options, "--bind", b)
-
 		}
 		options = append(options, "-D", cmd.Chroot)
 		options = append(options, cmdline...)
@@ -251,15 +256,19 @@ func (cmd Command) Run(label string, cmdline ...string) error {
 
 	defer w.flush()
 
-	if len(cmd.extraEnv) > 0 && cmd.ChrootMethod != CHROOT_METHOD_NSPAWN {
+	if len(cmd.extraEnv) > 0 && cmd.ChrootMethod != ChrootMethodNspawn {
 		exe.Env = append(os.Environ(), cmd.extraEnv...)
 	}
 
 	// Disable services start/stop for commands running in chroot
-	if cmd.ChrootMethod != CHROOT_METHOD_NONE {
+	if cmd.ChrootMethod != ChrootMethodNone {
 		services := ServiceHelper{cmd.Chroot}
-		services.Deny()
-		defer services.Allow()
+		if err := services.Deny(); err != nil {
+			return err
+		}
+		defer func() {
+			_ = services.Allow()
+		}()
 	}
 
 	// Save the original resolv.conf and copy version from host
@@ -285,11 +294,11 @@ type qemuHelper struct {
 	qemutarget string
 }
 
-func newQemuHelper(c Command) qemuHelper {
+func newQemuHelper(c Command) (*qemuHelper, error) {
 	q := qemuHelper{}
 
 	if c.Chroot == "" || c.Architecture == "" {
-		return q
+		return &q, nil
 	}
 
 	switch c.Architecture {
@@ -323,15 +332,19 @@ func newQemuHelper(c Command) qemuHelper {
 		if runtime.GOARCH != "amd64" {
 			q.qemusrc = "/usr/bin/qemu-x86_64-static"
 		}
+	case "sh4":
+		if runtime.GOARCH != "sh4" {
+			q.qemusrc = "/usr/bin/qemu-sh4-static"
+		}
 	default:
-		log.Panicf("Don't know qemu for Architecture %s", c.Architecture)
+		return nil, fmt.Errorf("unsupported qemu architecture %s", c.Architecture)
 	}
 
 	if q.qemusrc != "" {
 		q.qemutarget = path.Join(c.Chroot, q.qemusrc)
 	}
 
-	return q
+	return &q, nil
 }
 
 func (q qemuHelper) Setup() error {
